@@ -1,4 +1,4 @@
-/* $Id: lexer.c,v 1.147 2005/12/31 17:17:12 relson Exp $ */
+/* $Id: lexer.c 6492 2006-06-20 22:10:41Z relson $ */
 
 /**
  * \file lexer.c
@@ -330,10 +330,10 @@ int yyinput(byte *buf, size_t used, size_t size)
 	if (used < 1000 || used < size * 10)
 	    break;
 
-	if (count >= MAXTOKENLEN * 2 && 
+	if (count >= MAX_TOKEN_LEN * 2 && 
 	    long_token(buff.t.text, (uint) count)) {
 	    uint start = buff.t.leng - count;
-	    uint length = count - MAXTOKENLEN;
+	    uint length = count - max_token_len;
 	    buff_shift(&buff, start, length);
 	    count = buff.t.leng;
 	}
