@@ -1,4 +1,4 @@
-/* $Id: sighandler.c,v 1.7 2006/07/03 00:21:29 relson Exp $ */
+/* $Id: sighandler.c 6657 2007-03-22 15:24:47Z m-a $ */
 
 /*****************************************************************************
 
@@ -21,7 +21,7 @@ AUTHOR:
 
 /* Global Definitions */
 
-bool fDie = false;
+sig_atomic_t fDie = false;
 
 /* Function Definitions */
 
@@ -49,6 +49,10 @@ static void mysigdie(int sig)
 	fDie = true;
     else
 	exit(EX_ERROR);
+    /* XXX FIXME: Need something that is async-signal-safe here!
+
+       Note that _exit bypasses atexit, so we may see database
+       corruption! */
 }
 
 void signal_setup(void)
